package sun.security.krb5;

import sun.security.krb5.internal.KdcErrException;
import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.internal.KrbApErrException;
import sun.security.krb5.internal.KRBError;
import sun.security.util.DerValue;
import sun.security.krb5.internal.SeqNumber;
import java.io.IOException;
import sun.security.krb5.internal.LocalSeqNumber;
import sun.security.krb5.internal.APRep;
import sun.security.krb5.internal.EncAPRepPart;

public class KrbApRep
{
    private byte[] obuf;
    private byte[] ibuf;
    private EncAPRepPart encPart;
    private APRep apRepMessg;
    
    public KrbApRep(KrbApReq krbApReq, boolean b, EncryptionKey encryptionKey) throws KrbException, IOException {
        this.init(krbApReq, encryptionKey, new LocalSeqNumber());
    }
    
    public KrbApRep(byte[] array, Credentials credentials, KrbApReq krbApReq) throws KrbException, IOException {
        this(array, credentials);
        this.authenticate(krbApReq);
    }
    
    private void init(KrbApReq krbApReq, EncryptionKey encryptionKey, SeqNumber seqNumber) throws KrbException, IOException {
        this.createMessage(krbApReq.getCreds().key, krbApReq.getCtime(), krbApReq.cusec(), encryptionKey, seqNumber);
        this.obuf = this.apRepMessg.asn1Encode();
    }
    
    private KrbApRep(byte[] encoding, Credentials credentials) throws KrbException, IOException {
        this(new DerValue(encoding), credentials);
    }
    
    private KrbApRep(DerValue encoding, Credentials credentials) throws KrbException, IOException {
        APRep apRep;
        try {
            apRep = new APRep(encoding);
        }
        catch (Asn1Exception cause) {
            KRBError krbError = new KRBError(encoding);
            String errorString = krbError.getErrorString();
            String substring;
            if (errorString.charAt(errorString.length() - 1) == '\0') {
                substring = errorString.substring(0, errorString.length() - 1);
            }
            else {
                substring = errorString;
            }
            KrbException o = new KrbException(krbError.getErrorCode(), substring);
            o.initCause(cause);
            throw o;
        }
        encoding = new DerValue(apRep.encPart.reset(apRep.encPart.decrypt(credentials.key, 12)));
        this.encPart = new EncAPRepPart(encoding);
    }
    
    private void authenticate(KrbApReq krbApReq) throws KrbException, IOException {
        if (this.encPart.ctime.getSeconds() != krbApReq.getCtime().getSeconds() || this.encPart.cusec != krbApReq.getCtime().getMicroSeconds()) {
            throw new KrbApErrException(46);
        }
        return;
    }
    
    public EncryptionKey getSubKey() {
        return this.encPart.getSubKey();
    }
    
    public Integer getSeqNumber() {
        return this.encPart.getSeqNumber();
    }
    
    public byte[] getMessage() {
        return this.obuf;
    }
    
    private void createMessage(EncryptionKey encryptionKey, KerberosTime new_ctime, int new_cusec, EncryptionKey new_subKey, SeqNumber seqNumber) throws Asn1Exception, IOException, KdcErrException, KrbCryptoException {
        Integer new_seqNumber = null;
        if (seqNumber != null) {
            new_seqNumber = new Integer(seqNumber.current());
        }
        this.encPart = new EncAPRepPart(new_ctime, new_cusec, new_subKey, new_seqNumber);
        this.apRepMessg = new APRep(new EncryptedData(encryptionKey, this.encPart.asn1Encode(), 12));
    }
}
